@@ -40,10 +40,12 @@
 #include "pppsetting.h"
 #include "pppoesetting.h"
 #include "serialsetting.h"
+#include "tunsetting.h"
 #include "vlansetting.h"
 #include "vpnsetting.h"
 #include "wimaxsetting.h"
 
+#undef signals
 #if NM_CHECK_VERSION(1, 0, 0)
 #include <libnm/NetworkManager.h>
 #else
@@ -62,30 +64,53 @@
 #include <nm-setting-wimax.h>
 #include <nm-setting-wired.h>
 #include <nm-setting-wireless.h>
-
-#if NM_CHECK_VERSION(0, 9, 10)
 #include <nm-setting-team.h>
 #include <nm-setting-generic.h>
-#endif
+#define signals Q_SIGNALS
+
+#define NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY "autoconnect-priority"
+#define NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES "autoconnect-slaves"
 
 #endif
 
-#if NM_CHECK_VERSION(0, 9, 10)
+#if !NM_CHECK_VERSION(1, 0, 6)
+#define NM_SETTING_CONNECTION_METERED "metered"
+#endif
+
+#if !NM_CHECK_VERSION(1, 2, 0)
+#define NM_SETTING_TUN_SETTING_NAME "tun"
+#define NM_SETTING_CONNECTION_LLDP "lldp"
+#endif
+
+#if !NM_CHECK_VERSION(1, 4, 0)
+#define NM_SETTING_CONNECTION_STABLE_ID "stable-id"
+#endif
+
+#if !NM_CHECK_VERSION(1, 6, 0)
+#define NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES "autoconnect-retries"
+#endif
+
 #include "teamsetting.h"
 #include "genericsetting.h"
-#endif
 
 #include <QtCore/QUuid>
 
+namespace NetworkManager {
+    extern int checkVersion(const int x, const int y, const int z);
+}
+
 NetworkManager::ConnectionSettingsPrivate::ConnectionSettingsPrivate(ConnectionSettings *q)
     : name(NM_SETTING_CONNECTION_SETTING_NAME)
     , uuid(QUuid().toString())
     , type(ConnectionSettings::Wired)
     , autoconnect(true)
     , readOnly(false)
-#if NM_CHECK_VERSION(1, 0, 0)
+    , gatewayPingTimeout(0)
     , autoconnectPriority(0)
-#endif
+    , autoconnectRetries(-1)
+    , autoconnectSlaves(ConnectionSettings::SlavesDefault)
+    , lldp(ConnectionSettings::LldpDefault)
+    , metered(ConnectionSettings::MeteredUnknown)
     , q_ptr(q)
 { }
 
@@ -132,17 +157,17 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(NMBluetoothCapabili
     case ConnectionSettings::Cdma:
         addSetting(Setting::Ptr(new CdmaSetting()));
         addSetting(Setting::Ptr(new Ipv4Setting()));
-#if NM_CHECK_VERSION(1, 0, 0)
-        addSetting(Setting::Ptr(new Ipv6Setting()));
-#endif
+        if (NetworkManager::checkVersion(1, 0, 0)) {
+            addSetting(Setting::Ptr(new Ipv6Setting()));
+        }
         addSetting(Setting::Ptr(new PppSetting()));
         break;
     case ConnectionSettings::Gsm:
         addSetting(Setting::Ptr(new GsmSetting()));
         addSetting(Setting::Ptr(new Ipv4Setting()));
-#if NM_CHECK_VERSION(1, 0, 0)
-        addSetting(Setting::Ptr(new Ipv6Setting()));
-#endif
+        if (NetworkManager::checkVersion(1, 0, 0)) {
+            addSetting(Setting::Ptr(new Ipv6Setting()));
+        }
         addSetting(Setting::Ptr(new PppSetting()));
         break;
     case ConnectionSettings::Infiniband:
@@ -190,7 +215,6 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(NMBluetoothCapabili
         addSetting(Setting::Ptr(new WirelessSetting()));
         addSetting(Setting::Ptr(new WirelessSecuritySetting()));
         break;
-#if NM_CHECK_VERSION(0, 9, 10)
     case ConnectionSettings::Team:
         addSetting(Setting::Ptr(new TeamSetting()));
         addSetting(Setting::Ptr(new Ipv4Setting()));
@@ -201,7 +225,11 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(NMBluetoothCapabili
         addSetting(Setting::Ptr(new Ipv4Setting()));
         addSetting(Setting::Ptr(new Ipv6Setting()));
         break;
-#endif
+    case ConnectionSettings::Tun:
+        addSetting(Setting::Ptr(new TunSetting()));
+        addSetting(Setting::Ptr(new Ipv4Setting()));
+        addSetting(Setting::Ptr(new Ipv6Setting()));
+        break;
     case ConnectionSettings::Unknown:
     default:
         break;
@@ -228,9 +256,9 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(const NetworkManage
     case ConnectionSettings::Bluetooth:
         addSetting(connectionSettings->setting(Setting::Bluetooth));
         addSetting(connectionSettings->setting(Setting::Ipv4));
-#if NM_CHECK_VERSION(1, 0, 0)
-        addSetting(connectionSettings->setting(Setting::Ipv6));
-#endif
+        if (NetworkManager::checkVersion(1, 0, 0)) {
+            addSetting(connectionSettings->setting(Setting::Ipv6));
+        }
         if (q->setting(Setting::Gsm) && q->setting(Setting::Ppp) && q->setting(Setting::Serial)) {
             addSetting(connectionSettings->setting(Setting::Gsm));
             addSetting(connectionSettings->setting(Setting::Ppp));
@@ -245,17 +273,17 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(const NetworkManage
     case ConnectionSettings::Cdma:
         addSetting(connectionSettings->setting(Setting::Cdma));
         addSetting(connectionSettings->setting(Setting::Ipv4));
-#if NM_CHECK_VERSION(1, 0, 0)
-        addSetting(connectionSettings->setting(Setting::Ipv6));
-#endif
+        if (NetworkManager::checkVersion(1, 0, 0)) {
+            addSetting(connectionSettings->setting(Setting::Ipv6));
+        }
         addSetting(connectionSettings->setting(Setting::Ppp));
         break;
     case ConnectionSettings::Gsm:
         addSetting(connectionSettings->setting(Setting::Gsm));
         addSetting(connectionSettings->setting(Setting::Ipv4));
-#if NM_CHECK_VERSION(1, 0, 0)
-        addSetting(connectionSettings->setting(Setting::Ipv6));
-#endif
+        if (NetworkManager::checkVersion(1, 0, 0)) {
+            addSetting(connectionSettings->setting(Setting::Ipv6));
+        }
         addSetting(connectionSettings->setting(Setting::Ppp));
         break;
     case ConnectionSettings::Infiniband:
@@ -303,7 +331,6 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(const NetworkManage
         addSetting(connectionSettings->setting(Setting::Wireless));
         addSetting(connectionSettings->setting(Setting::WirelessSecurity));
         break;
-#if NM_CHECK_VERSION(0, 9, 10)
     case ConnectionSettings::Team:
         addSetting(connectionSettings->setting(Setting::Team));
         addSetting(connectionSettings->setting(Setting::Ipv4));
@@ -314,7 +341,11 @@ void NetworkManager::ConnectionSettingsPrivate::initSettings(const NetworkManage
         addSetting(connectionSettings->setting(Setting::Ipv4));
         addSetting(connectionSettings->setting(Setting::Ipv6));
         break;
-#endif
+    case ConnectionSettings::Tun:
+        addSetting(connectionSettings->setting(Setting::Tun));
+        addSetting(connectionSettings->setting(Setting::Ipv4));
+        addSetting(connectionSettings->setting(Setting::Ipv6));
+        break;
     case ConnectionSettings::Unknown:
     default:
         break;
@@ -353,12 +384,12 @@ NetworkManager::ConnectionSettings::ConnectionType NetworkManager::ConnectionSet
         type = Wired;
     } else if (typeString == QLatin1String(NM_SETTING_WIRELESS_SETTING_NAME)) {
         type = Wireless;
-#if NM_CHECK_VERSION(0, 9, 10)
     } else if (typeString == QLatin1String(NM_SETTING_TEAM_SETTING_NAME)) {
         type = Team;
     } else if (typeString == QLatin1String(NM_SETTING_GENERIC_SETTING_NAME)) {
         type = Generic;
-#endif
+    } else if (typeString == QLatin1String(NM_SETTING_TUN_SETTING_NAME)) {
+        type = Tun;
     }
 
     return type;
@@ -411,14 +442,15 @@ QString NetworkManager::ConnectionSettings::typeAsString(NetworkManager::Connect
     case Wireless:
         typeString = QLatin1String(NM_SETTING_WIRELESS_SETTING_NAME);
         break;
-#if NM_CHECK_VERSION(0, 9, 10)
     case Team:
         typeString = QLatin1String(NM_SETTING_TEAM_SETTING_NAME);
         break;
     case Generic:
         typeString = QLatin1String(NM_SETTING_GENERIC_SETTING_NAME);
         break;
-#endif
+    case Tun:
+        typeString = QLatin1String(NM_SETTING_TUN_SETTING_NAME);
+        break;
     default:
         break;
     };
@@ -448,23 +480,23 @@ NetworkManager::ConnectionSettings::ConnectionSettings(const NetworkManager::Con
 
     setId(other->id());
     setUuid(other->uuid());
-#if NM_CHECK_VERSION(0, 9, 10)
     setInterfaceName(other->interfaceName());
-#endif
     setConnectionType(other->connectionType());
     setPermissions(other->permissions());
     setAutoconnect(other->autoconnect());
-#if NM_CHECK_VERSION(1, 0, 0)
     setAutoconnectPriority(other->autoconnectPriority());
-#endif
     setTimestamp(other->timestamp());
     setReadOnly(other->readOnly());
     setZone(other->zone());
     setMaster(other->master());
     setSlaveType(other->slaveType());
-#if NM_CHECK_VERSION(0, 9, 10)
     setGatewayPingTimeout(other->gatewayPingTimeout());
-#endif
+    setAutoconnectRetries(other->autoconnectRetries());
+    setAutoconnectSlaves(other->autoconnectSlaves());
+    setLldp(other->lldp());
+    setMetered(other->metered());
+    setStableId(other->stableId());
+
     d->initSettings(other);
 }
 
@@ -489,11 +521,9 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
     setUuid(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_UUID)).toString());
     setConnectionType(typeFromString(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_TYPE)).toString()));
 
-#if NM_CHECK_VERSION(0, 9, 10)
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_INTERFACE_NAME))) {
         setInterfaceName(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_INTERFACE_NAME)).toString());
     }
-#endif
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_PERMISSIONS))) {
         QStringList permissions = connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_PERMISSIONS)).toStringList();
         Q_FOREACH (const QString & permission, permissions) {
@@ -506,11 +536,9 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
         setAutoconnect(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT)).toBool());
     }
 
-#if NM_CHECK_VERSION(1, 0, 0)
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY))) {
         setAutoconnectPriority(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY)).toInt());
     }
-#endif
 
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_TIMESTAMP))) {
         const int timestamp = connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_TIMESTAMP)).toInt();
@@ -539,11 +567,29 @@ void NetworkManager::ConnectionSettings::fromMap(const NMVariantMapMap &map)
         setSecondaries(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_SECONDARIES)).toStringList());
     }
 
-#if NM_CHECK_VERSION(0, 9, 10)
     if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_GATEWAY_PING_TIMEOUT))) {
         setGatewayPingTimeout(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_GATEWAY_PING_TIMEOUT)).toUInt());
     }
-#endif
+
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES))) {
+        setAutoconnectRetries(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES)).toInt());
+    }
+
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES))) {
+        setAutoconnectSlaves((NetworkManager::ConnectionSettings::AutoconnectSlaves)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES)).toInt());
+    }
+
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_LLDP))) {
+        setLldp((NetworkManager::ConnectionSettings::Lldp)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_LLDP)).toInt());
+    }
+
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_METERED))) {
+        setMetered((NetworkManager::ConnectionSettings::Metered)connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_METERED)).toInt());
+    }
+
+    if (connectionSettings.contains(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID))) {
+        setStableId(connectionSettings.value(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID)).toString());
+    }
 
     Q_FOREACH (const Setting::Ptr & setting, settings()) {
         if (map.contains(setting->name())) {
@@ -572,11 +618,9 @@ NMVariantMapMap NetworkManager::ConnectionSettings::toMap() const
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_TYPE), typeAsString(connectionType()));
     }
 
-#if NM_CHECK_VERSION(0, 9, 10)
     if (!interfaceName().isEmpty()) {
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_INTERFACE_NAME), interfaceName());
     }
-#endif
 
     if (!permissions().isEmpty()) {
         QStringList perm;
@@ -595,11 +639,9 @@ NMVariantMapMap NetworkManager::ConnectionSettings::toMap() const
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT), autoconnect());
     }
 
-#if NM_CHECK_VERSION(1, 0, 0)
     if (autoconnectPriority()) {
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY), autoconnectPriority());
     }
-#endif
 
     if (timestamp().isValid()) {
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_TIMESTAMP), timestamp().toTime_t());
@@ -625,11 +667,21 @@ NMVariantMapMap NetworkManager::ConnectionSettings::toMap() const
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_SECONDARIES), secondaries());
     }
 
-#if NM_CHECK_VERSION(0, 9, 10)
     if (gatewayPingTimeout()) {
         connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_GATEWAY_PING_TIMEOUT), gatewayPingTimeout());
     }
-#endif
+
+    if (autoconnectRetries() >= 0) {
+        connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES), autoconnectRetries());
+    }
+
+    connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES), autoconnectSlaves());
+    connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_LLDP), lldp());
+    connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_METERED), metered());
+
+    if (!stableId().isEmpty()) {
+        connectionSetting.insert(QLatin1String(NM_SETTING_CONNECTION_STABLE_ID), stableId());
+    }
 
     result.insert(QLatin1String(NM_SETTING_CONNECTION_SETTING_NAME), connectionSetting);
 
@@ -677,7 +729,6 @@ QString NetworkManager::ConnectionSettings::uuid() const
     return d->uuid;
 }
 
-#if NM_CHECK_VERSION(0, 9, 10)
 void NetworkManager::ConnectionSettings::setInterfaceName(const QString &interfaceName)
 {
     Q_D(ConnectionSettings);
@@ -691,7 +742,6 @@ QString NetworkManager::ConnectionSettings::interfaceName() const
 
     return d->interfaceName;
 }
-#endif
 
 void NetworkManager::ConnectionSettings::setConnectionType(NetworkManager::ConnectionSettings::ConnectionType type, NMBluetoothCapabilities bt_cap)
 {
@@ -743,7 +793,6 @@ bool NetworkManager::ConnectionSettings::autoconnect() const
     return d->autoconnect;
 }
 
-#if NM_CHECK_VERSION(1, 0, 0)
 void NetworkManager::ConnectionSettings::setAutoconnectPriority(int priority)
 {
     Q_D(ConnectionSettings);
@@ -757,7 +806,6 @@ int NetworkManager::ConnectionSettings::autoconnectPriority() const
 
     return d->autoconnectPriority;
 }
-#endif
 
 void NetworkManager::ConnectionSettings::setTimestamp(const QDateTime &timestamp)
 {
@@ -850,7 +898,6 @@ QStringList NetworkManager::ConnectionSettings::secondaries() const
     return d->secondaries;
 }
 
-#if NM_CHECK_VERSION(0, 9, 10)
 void NetworkManager::ConnectionSettings::setGatewayPingTimeout(quint32 timeout)
 {
     Q_D(ConnectionSettings);
@@ -864,7 +911,76 @@ quint32 NetworkManager::ConnectionSettings::gatewayPingTimeout() const
 
     return d->gatewayPingTimeout;
 }
-#endif
+
+int NetworkManager::ConnectionSettings::autoconnectRetries() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->autoconnectRetries;
+}
+
+void NetworkManager::ConnectionSettings::setAutoconnectRetries(int retries)
+{
+    Q_D(ConnectionSettings);
+
+    d->autoconnectRetries = retries;
+}
+
+NetworkManager::ConnectionSettings::AutoconnectSlaves NetworkManager::ConnectionSettings::autoconnectSlaves() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->autoconnectSlaves;
+}
+
+void NetworkManager::ConnectionSettings::setAutoconnectSlaves(NetworkManager::ConnectionSettings::AutoconnectSlaves autoconnectSlaves)
+{
+    Q_D(ConnectionSettings);
+
+    d->autoconnectSlaves = autoconnectSlaves;
+}
+
+NetworkManager::ConnectionSettings::Lldp NetworkManager::ConnectionSettings::lldp() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->lldp;
+}
+
+void NetworkManager::ConnectionSettings::setLldp(NetworkManager::ConnectionSettings::Lldp lldp)
+{
+    Q_D(ConnectionSettings);
+
+    d->lldp = lldp;
+}
+
+NetworkManager::ConnectionSettings::Metered NetworkManager::ConnectionSettings::metered() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->metered;
+}
+
+void NetworkManager::ConnectionSettings::setMetered(NetworkManager::ConnectionSettings::Metered metered)
+{
+    Q_D(ConnectionSettings);
+
+    d->metered = metered;
+}
+
+QString NetworkManager::ConnectionSettings::stableId() const
+{
+    Q_D(const ConnectionSettings);
+
+    return d->stableId;
+}
+
+void NetworkManager::ConnectionSettings::setStableId(const QString &stableId)
+{
+    Q_D(ConnectionSettings);
+
+    d->stableId = stableId;
+}
 
 NetworkManager::Setting::Ptr NetworkManager::ConnectionSettings::setting(Setting::SettingType type) const
 {
@@ -896,24 +1012,23 @@ QDebug NetworkManager::operator <<(QDebug dbg, const NetworkManager::ConnectionS
 
     dbg.nospace() << NM_SETTING_CONNECTION_ID << ": " << setting.id() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_UUID << ": " << setting.uuid() << '\n';
-#if NM_CHECK_VERSION(0, 9, 10)
     dbg.nospace() << NM_SETTING_CONNECTION_INTERFACE_NAME << ": " << setting.interfaceName() << '\n';
-#endif
     dbg.nospace() << NM_SETTING_CONNECTION_TYPE << ": " << setting.typeAsString(setting.connectionType()) << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_PERMISSIONS << ": " << setting.permissions() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_AUTOCONNECT << ": " << setting.autoconnect() << '\n';
-#if NM_CHECK_VERSION(1, 0, 0)
     dbg.nospace() << NM_SETTING_CONNECTION_AUTOCONNECT_PRIORITY << ": " << setting.autoconnectPriority() << '\n';
-#endif
     dbg.nospace() << NM_SETTING_CONNECTION_TIMESTAMP << ": " << setting.timestamp().toTime_t() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_READ_ONLY << ": " << setting.readOnly() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_ZONE << ": " << setting.zone() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_MASTER << ": " << setting.master() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_SLAVE_TYPE << ": " << setting.slaveType() << '\n';
     dbg.nospace() << NM_SETTING_CONNECTION_SECONDARIES << ": " << setting.secondaries() << '\n';
-#if NM_CHECK_VERSION(0, 9, 10)
     dbg.nospace() << NM_SETTING_CONNECTION_GATEWAY_PING_TIMEOUT << ": " << setting.gatewayPingTimeout() << '\n';
-#endif
+    dbg.nospace() << NM_SETTING_CONNECTION_AUTOCONNECT_RETRIES << ": " << setting.autoconnectRetries() << '\n';
+    dbg.nospace() << NM_SETTING_CONNECTION_AUTOCONNECT_SLAVES << ": " << setting.autoconnectSlaves() << '\n';
+    dbg.nospace() << NM_SETTING_CONNECTION_LLDP << ": " << setting.lldp() << '\n';
+    dbg.nospace() << NM_SETTING_CONNECTION_METERED << ": " << setting.metered() << '\n';
+    dbg.nospace() << NM_SETTING_CONNECTION_STABLE_ID << ": " << setting.stableId() << '\n';
     dbg.nospace() << "===================\n";
     Q_FOREACH (const Setting::Ptr & settingPtr, setting.settings()) {
         dbg.nospace() << settingPtr->typeAsString(settingPtr->type()).toUpper() << " SETTINGS\n";
@@ -933,6 +1048,7 @@ QDebug NetworkManager::operator <<(QDebug dbg, const NetworkManager::ConnectionS
             break;
         case Setting::BridgePort:
             dbg.nospace() << *(settingPtr.staticCast<NetworkManager::BridgePortSetting>().data());
+            break;
         case Setting::Cdma:
             dbg.nospace() << *(settingPtr.staticCast<NetworkManager::CdmaSetting>().data());
             break;
@@ -981,11 +1097,12 @@ QDebug NetworkManager::operator <<(QDebug dbg, const NetworkManager::ConnectionS
         case Setting::WirelessSecurity:
             dbg.nospace() << *(settingPtr.staticCast<NetworkManager::WirelessSecuritySetting>().data());
             break;
-#if NM_CHECK_VERSION(0, 9, 10)
         case Setting::Team:
             dbg.nospace() << *(settingPtr.staticCast<NetworkManager::TeamSetting>().data());
             break;
-#endif
+        case Setting::Tun:
+            dbg.nospace() << *(settingPtr.staticCast<NetworkManager::TunSetting>().data());
+            break;
         default:
             dbg.nospace() << *settingPtr.data();
         }
