@@ -38,6 +38,8 @@ public:
     QString username;
     NMStringMap data;
     NMStringMap secrets;
+    bool persistent;
+    uint timeout;
 };
 
 }
